/********************************************************************
 * Includes
*/
#include <zephyr/types.h>
#include <stddef.h>
#include <sys/printk.h>
#include <sys/util.h>
#include <stdio.h>
#include <inttypes.h>

//Sensor
#include <zephyr.h>
#include <device.h>
#include <drivers/sensor.h>

//GPIO
#include <devicetree.h>
#include <drivers/gpio.h>

//Power Management
#include <pm/pm.h>
#include <pm/device.h>
#include <pm/policy.h>


//Rotation 
#include "IMU/IMU.h"

// Pressure
#include "Pressure/Pressure.h"



/********************************************************************
 * Defines (Name, Zeiten)
*/

// names
#define DEVICE_NAME CONFIG_BT_DEVICE_NAME
#define DEVICE_NAME_LEN (sizeof(DEVICE_NAME) - 1)
//#define DEVICE_NAME_IMU invensense_mpu9250
#define LSM6DSO DT_INST(0, st_lsm6dso)

#if DT_NODE_HAS_STATUS(LSM6DSO, okay)
#define DEVICE_NAME_IMU DT_LABEL(LSM6DSO)
#else
#error Your devicetree has no enabled nodes with compatible "st,lsm6dso"
#define DEVICE_NAME_IMU "<none>"
#endif

#define DEVICE_NAME_PRESSURE meas_ms5607
#define GPIO_DEV_NAME1 "GPIO_0"
#define GPIO_I2C_DEV_NAME "i2c1_gpio0"
#define VDD_PIN_IMU 2			// Version 1 Power Pin = 16	// nRF52DK Pin = 2 
#define VDD_PIN_PRESSURE 23		// Version 1 Power Pin = 20 // nRF52DK Pin = 23


// measurement timings in ms
#define MEASURE_INTERVAL 2000 

//Devices 
const struct device *gpio = NULL;
const struct device *pressure = NULL;
const struct device *imu = NULL;
const struct device *gpio_i2c = NULL;

//Power Management
bool imu_power = true;
bool pressure_power = true;


/********************************************************************
 * get Runtime in seconds
*/
static const char *now_str(void)
{
	static char buf[16]; /* ...HH:MM:SS.MMM */
	uint32_t now = k_uptime_get_32();
	unsigned int ms = now % MSEC_PER_SEC;
	unsigned int s;
	unsigned int min;
	unsigned int h;

	now /= MSEC_PER_SEC;
	s = now % 60U;
	now /= 60U;
	min = now % 60U;
	now /= 60U;
	h = now;

	snprintf(buf, sizeof(buf), "%u:%02u:%02u.%03u",
		 h, min, s, ms);
	return buf;
}

/********************************************************************
 * get Runtime in minutes 
*/
static uint32_t getRuntimeinMinutes(void)
{
	uint32_t runtimeMinutes = k_uptime_get_32() /1000 / 60;
	printf("[%s]:%u minutes\n", now_str(), runtimeMinutes);
	return runtimeMinutes;
}



// Print timestamp for debugging purposes
void timestamp_print(void)
{
	printf("[%s]:", now_str());
}


static void gpio_init(){
	int ret = 0;
	gpio = device_get_binding(GPIO_DEV_NAME1);

	if (!gpio) {
		printk("Cannot find %s!\n", GPIO_DEV_NAME1);
	}

	//Neccessary for GPIO Sensor Powering

	/*
	ret = gpio_pin_configure(gpio, VDD_PIN_IMU, GPIO_OUTPUT_ACTIVE);	
	ret = gpio_pin_configure(gpio, VDD_PIN_PRESSURE, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		printk("Cannot configure %s!\n", GPIO_DEV_NAME1);
	}
	
	gpio_pin_set(gpio, VDD_PIN_IMU, 1); 
	gpio_pin_set(gpio, VDD_PIN_PRESSURE, 1);
	*/
	printk("waiting 2 seconds for pins to get up\n");
	k_sleep(K_MSEC(1000));

}


/* initialisation - pressure sensor */
static void pressure_init(){

	k_sleep(K_MSEC(MEASURE_INTERVAL*2));
	//struct sensor_value oversampling_rate = { 8192, 0 };
	pressure = DEVICE_DT_GET_ANY(DEVICE_NAME_PRESSURE);

	if (pressure == NULL) {
		printk("Could not find pressure sensor, aborting test.\n");
		
	}
	if (!device_is_ready(pressure)) {
		printk("pressure sensor %s is not ready, aborting test.\n",
			pressure->name);
	}
}

/* initialisation - inertial measurement unit */
static void imu_init(){

	imu = device_get_binding(DEVICE_NAME_IMU);

	if (imu == NULL) {
		printk("Could not find IMU sensor, aborting test.\n");
		k_sleep(K_MSEC(3000));
		
	}
	if (!device_is_ready(imu)) {
		printk("IMU sensor %s is not ready, aborting test.\n",
			imu->name);
	}
	
	if(set_sampling_freq(imu) != 0){
		printf("sensor wurde initialisiert.");
	}
}





/****************************************************************
 * Telemetrie update cycle
*/
static void update(){ 
	


	//fetchPressure
	//gpio_pin_set(gpio, VDD_PIN_PRESSURE, 1);
	k_sleep(K_MSEC(20));
	fetchPressureSensor(pressure);
	//gpio_pin_set(gpio, VDD_PIN_PRESSURE, 0);


	//pressure
	timestamp_print();
	get_pressure();

	//temperature
	timestamp_print();
	get_temperature();
	
	
	//RPM
	timestamp_print();
	
	//gpio_pin_set(gpio, VDD_PIN_IMU, 1);
	k_sleep(K_MSEC(20));
	get_rotation(imu);
	//gpio_pin_set(gpio, VDD_PIN_IMU, 0);


	printk("----------------------------------\n");
}



/********************************************************************
 * Application main entry point
*/ 
void main(void)
{	int ret = 0;

	gpio = device_get_binding(GPIO_DEV_NAME1);

	if (!gpio) {
		printk("Cannot find %s!\n", GPIO_DEV_NAME1);
	}

	//Neccessary for GPIO Sensor Powering
	ret = gpio_pin_configure(gpio, VDD_PIN_IMU, GPIO_OUTPUT);	
	if (ret < 0) {
		printk("Cannot configure %s!\n", GPIO_DEV_NAME1);
	}
	ret = gpio_pin_configure(gpio, VDD_PIN_PRESSURE, GPIO_OUTPUT);
	if (ret < 0) {
		printk("Cannot configure %s!\n", GPIO_DEV_NAME1);
	}

	printk("waiting 2 seconds for pins to get up gpio initialisiert \n");
	k_sleep(K_MSEC(1000));
	
	gpio_pin_set(gpio, VDD_PIN_IMU, 1); 
	gpio_pin_set(gpio, VDD_PIN_PRESSURE, 1);
	
	printk("\n\n\n===============================================================\n");
	k_sleep(K_MSEC(1000));

	/* Initialize the pressure sensor */
	pressure_init();

	/* Initialize the IMU sensor */
	imu_init();


	/* Periodic update of sensor data and advertising packets */ 
	for(;;){
		update();								
		k_sleep(K_MSEC(MEASURE_INTERVAL));
	}	
}
